package com.study.libs;

import android.app.Application;
import android.content.pm.PackageInfo;
import android.content.pm.PackageManager;
import android.content.res.AssetManager;
import android.content.res.Resources;
import android.os.Build;
import android.text.TextUtils;

import androidx.annotation.RequiresApi;

import com.study.libs.utils.SkinResources;

import java.io.File;
import java.lang.reflect.Method;
import java.util.Observable;

/**
 * 外部入口
 * 同时作为一个被观察者，发送通知
 */

public class SkinManager extends Observable {

    private Application application;
    private static volatile SkinManager skinManager;

    @RequiresApi(api = Build.VERSION_CODES.KITKAT)
    private SkinManager(Application application) {
        this.application = application;
        SkinPreference.init(application.getApplicationContext());
        SkinResources.init(application);
        application.registerActivityLifecycleCallbacks(new ActivityLifeCycleCallback(this));
        loadSkin(SkinPreference.getInstance().getSkin());
    }

    @RequiresApi(api = Build.VERSION_CODES.KITKAT)
    public static SkinManager init(Application application) {
        if (null == skinManager) {
            synchronized (SkinManager.class) {
                if (null == skinManager) {
                    skinManager = new SkinManager(application);
                }
            }
        }
        return skinManager;
    }

    public static SkinManager getInstance() {
        return skinManager;
    }

    private void versionCheck() {
        if (Build.VERSION.SDK_INT > Build.VERSION_CODES.P) {
            throw new RuntimeException("版本大于9的系统不支持本功能");
        }
    }

    @RequiresApi(api = Build.VERSION_CODES.KITKAT)
    public void loadSkin(String skinPath) {
        if (TextUtils.isEmpty(skinPath) && !new File(skinPath).exists()) {//如果为空或者不存在该文件，则不用管
            SkinPreference.getInstance().reset();
            SkinResources.getInstance().reset();
        } else {
            try {
                Resources appResource = application.getResources();
                AssetManager assetManager = AssetManager.class.newInstance();
                Method method = assetManager.getClass().getDeclaredMethod("addAssetPath", String.class);
                method.invoke(assetManager, skinPath);

                Resources skinResources = new Resources(assetManager, appResource.getDisplayMetrics(), appResource.getConfiguration());
                PackageManager packageManager = application.getPackageManager();
                PackageInfo info = packageManager.getPackageArchiveInfo(skinPath, PackageManager.GET_ACTIVITIES);
                SkinResources.getInstance().applySkin(skinResources, info.packageName);
                SkinPreference.getInstance().setSkin(skinPath);

            } catch (Exception e) {
                e.printStackTrace();
            }
        }
        setChanged();
        notifyObservers();
    }

    /**
     * 重置一切
     */
    public void reset() {
        SkinPreference.getInstance().reset();
        SkinResources.getInstance().reset();
        setChanged();
        notifyObservers();
    }
}
