package com.study.libs.utils;

import android.content.Context;
import android.content.res.ColorStateList;
import android.content.res.Resources;
import android.graphics.drawable.Drawable;
import android.text.TextUtils;

import androidx.core.content.res.ResourcesCompat;

public class SkinResources {


    //皮肤包的包名
    private String skinPkgName;
    //是否使用默认的皮肤，用原始的内容
    private boolean isDefaultSkin = true;

    //app原始的resource
    private Resources appResources;
    //皮肤包的resources
    private Resources skinResources;

    private static volatile SkinResources instance;

    private SkinResources(Context context) {
        appResources = context.getResources();
    }

    public static void init(Context context) {
        if (null == instance) {
            synchronized (SkinResources.class) {
                if (null == instance) {
                    instance = new SkinResources(context);
                }
            }
        }
    }

    public static SkinResources getInstance() {
        return instance;
    }

    public void reset() {
        skinResources = null;
        skinPkgName = null;
        isDefaultSkin = true;
    }

    public void applySkin(Resources resources, String pkgName) {
        skinResources = resources;
        skinPkgName = pkgName;
        //是否使用默认皮肤
        isDefaultSkin = TextUtils.isEmpty(pkgName) || resources == null;
    }

    /**
     * 1. 通过原始app中的resId获取到自己的名字（R.color.icon_up，获取到的就是icon_up）和类型
     * 2. 根据名字和类型获取皮肤包中的ID
     */
    public int getIdentifier(int resId) {
        if (isDefaultSkin) {
            return resId;
        }
        //获取名称，如：icon_up
        String resName = appResources.getResourceEntryName(resId);
        //获取类型，如：drawable
        String resType = appResources.getResourceTypeName(resId);
        int skinId = skinResources.getIdentifier(resName, resType, skinPkgName);
        return skinId;
    }

    /**
     * 输入主app的id，获取皮肤包中的
     */
    public int getColor(int resId) {
        if (isDefaultSkin) {
            return appResources.getColor(resId);
        }
        int skinId = getIdentifier(resId);
        if (skinId == 0) {
            return appResources.getColor(resId);
        }
        return skinResources.getColor(skinId);
    }

    public Drawable getDrawable(int resId) {
        if (isDefaultSkin) {
            return ResourcesCompat.getDrawable(appResources, resId, null);
        }
        int skinId = getIdentifier(resId);
        if (skinId == 0) {
            return ResourcesCompat.getDrawable(appResources, resId, null);
        }
        return ResourcesCompat.getDrawable(skinResources, skinId, null);
    }

    public ColorStateList getColorStateList(int resId) {
        if (isDefaultSkin) {
            return ResourcesCompat.getColorStateList(appResources, resId, null);
        }
        int skinId = getIdentifier(resId);
        if (skinId == 0) {
            return ResourcesCompat.getColorStateList(appResources, resId, null);
        }
        return ResourcesCompat.getColorStateList(skinResources, skinId, null);
    }


    /**
     * 可能是Color 也可能是drawable
     *
     * @return
     */
    public Object getBackground(int resId) {
        String resourceTypeName = appResources.getResourceTypeName(resId);
        if ("color".equals(resourceTypeName)) {
            return getColor(resId);
        } else {
            // drawable
            return getDrawable(resId);
        }
    }

}
